import unittest

import ramda as R

"""
https://github.com/ramda/ramda/blob/master/test/equals.js
"""

a = []
b = a


class TestEquals(unittest.TestCase):
  def test_for_deep_equality_of_its_operands(self):
    self.assertEqual(True, R.equals(100, 100))
    self.assertEqual(False, R.equals(100, '100'))
    self.assertEqual(True, R.equals(a, b))

  def test_considers_equal_boolean_primitives_equal(self):
    self.assertEqual(True, R.equals(True, True))
    self.assertEqual(True, R.equals(False, False))
    self.assertEqual(False, R.equals(True, False))
    self.assertEqual(False, R.equals(False, True))

  def test_considers_equivalent_boolean_objects_equal(self):
    self.assertEqual(True, R.equals(bool(True), bool(True)))
    self.assertEqual(True, R.equals(bool(False), bool(False)))
    self.assertEqual(False, R.equals(bool(True), bool(False)))
    self.assertEqual(False, R.equals(bool(False), bool(True)))

  # NOTICE: python does not have bool object

  def test_bool_with_objects(self):
    class T:
      def __bool__(self):
        return True

    class F:
      def __bool__(self):
        return False
    t = T()
    f = F()
    self.assertEqual(True, R.equals(bool(t), True))
    self.assertEqual(False, R.equals(bool(t), False))
    self.assertEqual(True, R.equals(bool(f), False))
    self.assertEqual(False, R.equals(bool(f), True))

  def test_considers_equal_number_primitives_equal(self):
    self.assertEqual(True, R.equals(0, 0))
    self.assertEqual(False, R.equals(0, 1))
    self.assertEqual(False, R.equals(1, 0))

  def test_considers_equal_string_primitives_equal(self):
    self.assertEqual(True, R.equals('', ''))
    self.assertEqual(False, R.equals('', 'x'))
    self.assertEqual(False, R.equals('x', ''))
    self.assertEqual(True, R.equals('foo', 'foo'))
    self.assertEqual(False, R.equals('foo', 'bar'))
    self.assertEqual(False, R.equals('bar', 'foo'))

    # Test python style str
    self.assertEqual(True, R.equals('foo', "foo"))
    self.assertEqual(True, R.equals('foo', """foo"""))
    self.assertEqual(True, R.equals('foo', '''foo'''))
    self.assertEqual(True, R.equals("foo", """foo"""))
    self.assertEqual(True, R.equals("foo", '''foo'''))
    self.assertEqual(True, R.equals("""foo""", '''foo'''))
    self.assertEqual(True, R.equals('1', str(1)))
    self.assertEqual(True, R.equals("1", str(1)))
    self.assertEqual(True, R.equals('''1''', str(1)))
    self.assertEqual(True, R.equals("""1""", str(1)))

  def test_handle_dicts(self):
    self.assertEqual(True, R.equals({}, {}))
    self.assertEqual(True, R.equals({'a': 1, 'b': 2}, {'a': 1, 'b': 2}))
    self.assertEqual(True, R.equals({'a': 1, 'b': 2}, {'b': 2, 'a': 1}))
    self.assertEqual(False, R.equals({'a': 2, 'b': 2}, {'b': 2, 'a': 1}))
    self.assertEqual(False, R.equals({'a': 1, 'b': 2, 'c': 3}, {'b': 2, 'a': 1}))

  def test_considers_equivalent_arguments_objects_equal(self):
    a = (lambda *args: args)()
    b = (lambda *args: args)()
    c = (lambda *args: args)(1, 2, 3)
    d = (lambda *args: args)(1, 2, 3)

    self.assertEqual(True, R.equals(a, b))
    self.assertEqual(True, R.equals(b, a))
    self.assertEqual(True, R.equals(c, d))
    self.assertEqual(True, R.equals(d, c))
    self.assertEqual(False, R.equals(a, c))
    self.assertEqual(False, R.equals(c, a))

  def test_considers_equivalent_error_objects_equal(self):
    self.assertEqual(True, R.equals(ValueError('XXX'), ValueError('XXX')))
    self.assertEqual(False, R.equals(ValueError('XXX'), ValueError('YYY')))
    self.assertEqual(False, R.equals(IndexError('XXX'), ValueError('XXX')))
    self.assertEqual(False, R.equals(IndexError('XXX'), ValueError('YYY')))

  # regex is not built-in type in python, so we don't support it yet
  # TODO: support regex

  def test_handles_list(self):
    listA = [1, 2, 3]
    listB = [1, 3, 2]
    self.assertEqual(False, R.equals([], {}))
    self.assertEqual(False, R.equals(listA, listB))

  def test_handles_reacursieve_data_structures(self):
    c = {}
    c['v'] = c
    d = {}
    d['v'] = d
    e = []
    e.append(e)
    f = []
    f.append(f)
    nestA = {'a': [1, 2, {'c': 1}], 'b': 1}
    nestB = {'a': [1, 2, {'c': 1}], 'b': 1}
    nestC = {'a': [1, 2, {'c': 2}], 'b': 1}
    # Python can not compare such recursive data structure
    # self.assertEqual(True, R.equals(c, d))
    # self.assertEqual(True, R.equals(e, f))
    self.assertEqual(True, R.equals(nestA, nestB))
    self.assertEqual(False, R.equals(nestA, nestC))

    # date is not built-in type in python, so we don't support it yet
    # TODO: support date

  def test_compares_set_objects_by_value(self):
    self.assertEqual(True, R.equals(set(), set()))
    self.assertEqual(False, R.equals(set([]), set([1])))
    self.assertEqual(False, R.equals(set([]), set([1])))
    self.assertEqual(True, R.equals(set([1, 2]), set([2, 1])))
    # self.assertEqual(True, R.equals(set([1, set([2, set([3])])]), set(1, [set([2, set([3])])])))
    # self.assertEqual(True, R.equals(set([[1, 2, 3], [4, 5, 6]]), set([[4, 5, 6], [1, 2, 3]])))
    # self.assertEqual(True, R.equals(set([[1, 2, 3], [4, 5, 6]]), set([[4, 5, 6], [7, 8, 9]])))

  def test_compares_WeakMap_objects_by_identity(self):
    """
    ref: https://docs.python.org/3/library/weakref.html
    """
    class Dict(dict):
      pass
    d = Dict()
    self.assertEqual(True, R.equals(d, d))
    # TODO: catch up with weakref
    # TODO: WeakSet if need
    # self.assertEqual(False, R.equals(d, Dict()))

  def test_dispatches_to_equals_method_recursively(self):
    class Left:
      def __init__(self, x):
        self.value = x
      def equals(self, x):
        return isinstance(x, Left) and R.equals(self.value, x.value)

    class Right:
      def __init__(self, x):
        self.value = x
      def equals(self, x):
        return isinstance(x, Right) and R.equals(self.value, x.value)

    self.assertEqual(True, R.equals(Left([42]), Left([42])))
    self.assertEqual(False, R.equals(Left([42]), Left([43])))
    self.assertEqual(False, R.equals(Left(42), {'value': 42}))
    self.assertEqual(False, R.equals({'value': 42}, Left(42)))
    self.assertEqual(False, R.equals(Left(42), Right(42)))
    self.assertEqual(False, R.equals(Right(42), Left(42)))

    self.assertEqual(True, R.equals([Left(42)], [Left(42)]))
    self.assertEqual(False, R.equals([Left(42)], [Right(42)]))
    self.assertEqual(False, R.equals([Right(42)], [Left(42)]))
    self.assertEqual(True, R.equals([Right(42)], [Right(42)]))

  def test_is_commutative(self):
    class Point:
      def __init__(self, x, y):
        self.x = x
        self.y = y
      def equals(self, point):
        return isinstance(point, Point) and self.x == point.x and self.y == point.y

    class ColorPoint:
      def __init__(self, x, y, color):
        self.x = x
        self.y = y
        self.color = color
      def equals(self, point):
        return isinstance(point, ColorPoint) and self.x == point.x and self.y == point.y and self.color == point.color

    self.assertEqual(True, R.equals(Point(1, 2), Point(1, 2)))
    self.assertEqual(True, R.equals(ColorPoint(1, 2, 'red'), ColorPoint(1, 2, 'red')))
    self.assertEqual(False, R.equals(Point(1, 2), ColorPoint(1, 2, 'red')))
    self.assertEqual(False, R.equals(ColorPoint(1, 2, 'red'), Point(1, 2)))

  def test_clone(self):
    pass
    #TODO: clone

  def test_nan(self):
    self.assertEqual(True, R.equals(float('nan'), float('nan')))

if __name__ == '__main__':
  unittest.main()




import unittest

import ramda as R

"""
https://github.com/ramda/ramda/blob/master/test/toString.js
"""


class TestToString(unittest.TestCase):
  def test_returns_the_string_representation_of_None(self):
    self.assertEqual('None', R.toString(None))

  def test_returns_the_string_representation_of_a_Boolean_primitive(self):
    self.assertEqual('True', R.toString(True))
    self.assertEqual('False', R.toString(False))

  def test_returns_the_string_representation_of_a_number_primitive(self):
    self.assertEqual('0', R.toString(0))
    # TODO: -0
    self.assertEqual('1.23', R.toString(1.23))
    self.assertEqual('-1.23', R.toString(-1.23))
    self.assertEqual('1e+23', R.toString(1e+23))
    self.assertEqual('-1e+23', R.toString(-1e+23))
    self.assertEqual('1e-23', R.toString(1e-23))
    self.assertEqual('-1e-23', R.toString(-1e-23))
    self.assertEqual('inf', R.toString(float('inf')))
    self.assertEqual('-inf', R.toString(float('-inf')))
    self.assertEqual('nan', R.toString(float('nan')))

  def test_returns_the_string_representation_of_a_string_primitive(self):
    self.assertEqual('"abc"', R.toString('abc'))
    self.assertEqual('"x \\"y\\" z"', R.toString('x "y" z'))
    self.assertEqual('"\' \'"', R.toString("' '"))
    self.assertEqual('"\\" \\""', R.toString('" "'))
    self.assertEqual('"\\b \\b"', R.toString('\b \b'))
    self.assertEqual('"\\f \\f"', R.toString('\f \f'))
    self.assertEqual('"\\n \\n"', R.toString('\n \n'))
    self.assertEqual('"\\r \\r"', R.toString('\r \r'))
    self.assertEqual('"\\t \\t"', R.toString('\t \t'))
    self.assertEqual('"\\v \\v"', R.toString('\v \v'))
    self.assertEqual('"\\0 \\0"', R.toString('\0 \0'))
    self.assertEqual('"\\\\ \\\\"', R.toString('\\ \\'))

  # TODO: add other tests

if __name__ == '__main__':
  unittest.main()



import unittest

import ramda as R

from .helpers.listXf import listXf

"""
https://github.com/ramda/ramda/blob/master/test/any.js
"""


def odd(n): return n % 2 == 1
def T(): return True
intoArray = R.into([])


class TestAny(unittest.TestCase):
  def test_returns_true_if_any_elements_satisfy_the_predicate(self):
    self.assertEqual(True, R.any(odd, [2, 4, 6, 8, 10, 11, 12]))

  def test_returns_false_if_all_elements_fails_to_satisfy_the_predicate(self):
    self.assertEqual(False, R.any(odd, [2, 4, 6, 8, 10, 12]))

  def test_returns_true_into_array_if_any_elements_satisfy_the_predicate(self):
    self.assertEqual([True], intoArray(R.any(odd), [2, 4, 6, 8, 10, 11, 12]))

  def test_returns_false_if_all_elements_fails_to_satisfy_the_predicate(self):
    self.assertEqual([False], intoArray(R.any(odd), [2, 4, 6, 8, 10, 12]))

  def test_works_with_more_complex_objects(self):
    people = [{'first': 'Paul', 'last': 'Grenier'}, {'first': 'Mike', 'last': 'Hurley'}, {'first': 'Will', 'last': 'Klein'}]
    def alliterative(person): return person.get('first')[0] == person.get('last')[0]
    self.assertEqual(False, R.any(alliterative, people))
    people.append({'first': 'Scott', 'last': 'Sauyet'})
    self.assertEqual(True, R.any(alliterative, people))

  def test_returns_false_for_an_empty_list(self):
    self.assertEqual(False, R.any(T, []))

  def test_returns_false_into_array_for_an_empty_list(self):
    self.assertEqual([False], intoArray(R.any(T), []))

  def test_dispatches_when_given_a_transformer_in_list_position(self):
    res = R.any(odd, listXf)
    self.assertEqual(res.any, False)
    self.assertEqual(res.f, odd)
    self.assertEqual(res.xf, listXf)

  def test_can_act_as_a_transducer(self):
    input = [2, 4, 6, 8, 10]
    expected = [False]
    self.assertEqual(expected, R.into([], R.any(odd), input))
    # TODO: R.transduce


if __name__ == '__main__':
  unittest.main()



import unittest

import ramda as R

"""
https://github.com/ramda/ramda/blob/master/test/reverse.js
"""


class TestReverse(unittest.TestCase):
  def test_reverses_arrays(self):
    self.assertEqual([], R.reverse([]))
    self.assertEqual([1], R.reverse([1]))
    self.assertEqual([1, 2], R.reverse([2, 1]))
    self.assertEqual([1, 2, 3], R.reverse([3, 2, 1]))

  def test_reverses_twice_an_array_should_be_the_array_itself(self):
    self.assertEqual([1, 2, 3], R.reverse(R.reverse([1, 2, 3])))

  def test_reverses_strings(self):
    self.assertEqual('', R.reverse(''))
    self.assertEqual('1', R.reverse('1'))
    self.assertEqual('12', R.reverse('21'))
    self.assertEqual('123', R.reverse('321'))

  def test_reverses_twice_a_string_should_be_the_string_itself(self):
    self.assertEqual('123', R.reverse(R.reverse('123')))


if __name__ == '__main__':
  unittest.main()




import unittest

import ramda as R

"""
https://github.com/ramda/ramda/blob/master/test/reduce.js
"""


def add(a, b): return a + b
def mult(a, b): return a * b


def gen_int():
  yield 1
  yield 2
  yield 3
  yield 4


class ReduceObj:
  def reduce(self, step, acc):
    return 'override'


class TestReduce(unittest.TestCase):
  def test_folds_simple_functions_over_arrays_with_the_supplied_accumulator(self):
    self.assertEqual(10, R.reduce(add, 0, [1, 2, 3, 4]))
    self.assertEqual(24, R.reduce(mult, 1, [1, 2, 3, 4]))

  def test_dispatches_to_objects_that_implement_reduce(self):
    obj = ReduceObj()
    self.assertEqual('override', R.reduce(add, 0, obj))
    self.assertEqual('override', R.reduce(add, 10, obj))

  def test_returns_the_accumulator_for_an_empty_array(self):
    self.assertEqual(0, R.reduce(add, 0, []))
    self.assertEqual(1, R.reduce(mult, 1, []))
    self.assertEqual([], R.reduce(R.concat, [], []))

  def test_returns_the_accumulator_for_an_None_list(self):
    self.assertEqual(0, R.reduce(add, 0, None))
    self.assertEqual([], R.reduce(R.concat, [], None))

  def test_short_circuits_with_reduced(self):
    def addWithMaxOf10(acc, val): return R.reduced(acc) if acc + val > 10 else acc + val
    self.assertEqual(10, R.reduce(addWithMaxOf10, 0, [1, 2, 3, 4]))
    self.assertEqual(6, R.reduce(addWithMaxOf10, 0, [2, 4, 6, 8]))


class TestPythonStyleReduce(unittest.TestCase):
  def test_with_yield(self):
    self.assertEqual(10, R.reduce(add, 0, gen_int()))
    self.assertEqual(24, R.reduce(mult, 1, gen_int()))


if __name__ == '__main__':
  unittest.main()




import unittest

import ramda as R

"""
https://github.com/ramda/ramda/blob/master/test/props.js
"""

d = {
    'a': 1,
    'b': 2,
    'c': 3,
    'd': 4,
    'e': 5,
    'f': 6,
}


class TestPropsForDict(unittest.TestCase):
  def test_returns_empty_array_if_no_properties_requested(self):
    self.assertEqual([], R.props([], d))

  def test_returns_values_for_requested_properties(self):
    self.assertEqual([1, 5], R.props(['a', 'e'], d))

  def test_preserves_order(self):
    self.assertEqual([6, 3, 5], R.props(['f', 'c', 'e'], d))

  def test_returns_None_for_nonexistent_properties(self):
    ps = R.props(['a', 'nonexistent'], d)
    self.assertEqual(2, len(ps))
    self.assertEqual(1, ps[0])
    self.assertEqual(None, ps[1])

class Obj:
  def __init__(self, a, b, c, d, e, f):
    self.a = a
    self.b = b
    self.c = c
    self.d = d
    self.e = e
    self.f = f
obj = Obj(1, 2, 3, 4, 5, 6)

class TestPropsForObject(unittest.TestCase):
  def test_returns_empty_array_if_no_properties_requested(self):
    self.assertEqual([], R.props([], obj))

  def test_returns_values_for_requested_properties(self):
    self.assertEqual([1, 5], R.props(['a', 'e'], obj))

  def test_preserves_order(self):
    self.assertEqual([6, 3, 5], R.props(['f', 'c', 'e'], obj))

  def test_returns_None_for_nonexistent_properties(self):
    ps = R.props(['a', 'nonexistent'], obj)
    self.assertEqual(2, len(ps))
    self.assertEqual(1, ps[0])
    self.assertEqual(None, ps[1])

if __name__ == '__main__':
  unittest.main()




import unittest

import ramda as R

from .helpers.listXf import listXf

"""
https://github.com/ramda/ramda/blob/master/test/map.js
"""


def times2(x): return x * 2
def add1(x): return x + 1
def dec(x): return x - 1


class A:
  def __init__(self):
    self.a = 1


class B(A):
  def __init__(self):
    super().__init__()
    self.b = 2


class C:
  def __init__(self, x):
    self.x = x

  def map(self, f):
    return f(self.x)


class TestMap(unittest.TestCase):
  def test_maps_simple_functions_over_array(self):
    self.assertEqual([2, 4, 6, 8], R.map(times2, [1, 2, 3, 4]))

  def test_maps_over_objects(self):
    a, b = A(), B()
    mappedA = R.map(dec, a)
    mappedB = R.map(dec, b)
    self.assertEqual(0, mappedA.a)
    self.assertEqual(0, mappedB.a)
    self.assertEqual(1, mappedB.b)

    # make sure not change original objects
    self.assertEqual(1, a.a)
    self.assertEqual(1, b.a)
    self.assertEqual(2, b.b)

  def test_maps_over_dicts(self):
    self.assertEqual({}, R.map(dec, {}))
    self.assertEqual({'x': 3, 'y': 4, 'z': 5}, R.map(dec, {'x': 4, 'y': 5, 'z': 6}))

  def test_interprets_as_a_functor(self):
    def f(a): return a - 1
    def g(b): return b * 2
    h = R.map(f, g)
    self.assertEqual((10 * 2) - 1, h(10))

  def test_dispatches_to_objects_that_implement_map(self):
    c = C(42)
    self.assertEqual(41, R.map(dec, c))

  def test_dispatches_to_transformer_objects(self):
    o = R.map(add1, listXf)
    self.assertEqual(add1, o.f)
    self.assertEqual(listXf, o.xf)

  def test_throws_an_Exception_on_None(self):
    with self.assertRaises(Exception):
      R.map(times2, None)

  def test_composes(self):
    mdouble = R.map(times2)
    mdec = R.map(dec)
    self.assertEqual([19, 39, 59], mdec(mdouble([10, 20, 30])))

  def test_can_compose_transducer_style(self):
    mdouble = R.map(times2)
    mdec = R.map(dec)
    xcomp = mdec(mdouble(listXf))
    self.assertEqual(dec, xcomp.f)
    self.assertEqual(listXf, xcomp.xf.xf)
    self.assertEqual(times2, xcomp.xf.f)

  def test_can_act_as_a_transducer(self):
    self.assertEqual([2, 4, 6, 8], R.into([], R.map(times2), [1, 2, 3, 4]))
    # TODO: R.transduce


if __name__ == '__main__':
  unittest.main()



import unittest

import ramda as R

"""
https://github.com/ramda/ramda/blob/master/test/append.js
"""


class TestAppend(unittest.TestCase):
  def test_adds_the_element_to_the_end_of_the_list(self):
    self.assertEqual(['x', 'y', 'z'], R.append('z', ['x', 'y']))
    self.assertEqual(['x', 'y', ['a', 'z']], R.append(['a', 'z'], ['x', 'y']))

  def test_works_on_empty_list(self):
    self.assertEqual([1], R.append(1, []))


if __name__ == '__main__':
  unittest.main()



from .private._curry3 import _curry3
from .private._includesWith import _includesWith


def inner_differenceWith(pred, first, second):
  out = []
  idx = 0
  firstLength = len(first)
  while idx < firstLength:
    if (not _includesWith(pred, first[idx], second)) and (not _includesWith(pred, first[idx], out)):
      out.append(first[idx])
    idx += 1
  return out


differenceWith = _curry3(inner_differenceWith)




import unittest

import ramda as R
from ramda.private._isFunction import _isFunction

"""
https://github.com/ramda/ramda/blob/master/test/pluck.js
"""

people = [
    {'name': 'Fred', 'age': 23},
    {'name': 'Wilma', 'age': 21},
    {'name': 'Pebbles', 'age': 2},
]


class TestPluck(unittest.TestCase):
  def test_returns_a_function_that_maps_the_appropriate_property_over_an_array(self):
    nm = R.pluck('name')
    self.assertTrue(_isFunction(nm))
    self.assertEqual(['Fred', 'Wilma', 'Pebbles'], nm(people))

  def test_work_with_a_pure_dict(self):
    obj = {'a': {'val': 3}, 'b': {'val': 5}}
    self.assertEqual({'a': 3, 'b': 5}, R.pluck('val', obj))

  def test_works_with_objects(self):
    nm = R.pluck('name')

    class People:
      def __init__(self, name, age):
        self.name = name
        self.age = age
    fred = People('Fred', 23)
    wilma = People('Wilma', 21)
    pebbles = People('Pebbles', 2)
    self.assertEqual(['Fred', 'Wilma', 'Pebbles'], nm([fred, wilma, pebbles]))

  def test_behaves_as_a_transducer_when_given_a_transducer_in_list_position(self):
    numbers = [{'a': 1}, {'a': 2}, {'a': 3}, {'a': 4}]
    # TODO: transduce


if __name__ == '__main__':
  unittest.main()




import unittest
from random import randint

import ramda as R


def source(a, b, c):
  return a * b * c


'''
https://github.com/ramda/ramda/blob/master/test/curryN.js
'''


class TestCurryN(unittest.TestCase):

  def test_accepts_an_arity(self):
    curried = R.curryN(3, source)
    self.assertEqual(6, curried(1)(2)(3))
    self.assertEqual(6, curried(1, 2)(3))
    self.assertEqual(6, curried(1)(2, 3))
    self.assertEqual(6, curried(1, 2, 3))

  def test_can_be_partially_applied(self):
    curry3 = R.curryN(3)
    curried = curry3(source)
    self.assertEqual(6, curried(1)(2)(3))
    self.assertEqual(6, curried(1, 2)(3))
    self.assertEqual(6, curried(1)(2, 3))
    self.assertEqual(6, curried(1, 2, 3))

  def test_supports_R_placeholder(self):
    def f(*arguments):
      return list(arguments)
    g = R.curryN(3, f)
    _ = R.__

    self.assertEqual([1, 2, 3], g(1)(2)(3))
    self.assertEqual([1, 2, 3], g(1)(2, 3))
    self.assertEqual([1, 2, 3], g(1, 2)(3))
    self.assertEqual([1, 2, 3], g(1, 2, 3))

    self.assertEqual([1, 2, 3], g(_, 2, 3)(1))
    self.assertEqual([1, 2, 3], g(1, _, 3)(2))
    self.assertEqual([1, 2, 3], g(1, 2, _)(3))

    self.assertEqual([1, 2, 3], g(1, _, _)(2)(3))
    self.assertEqual([1, 2, 3], g(_, 2, _)(1)(3))
    self.assertEqual([1, 2, 3], g(_, _, 3)(1)(2))

    self.assertEqual([1, 2, 3], g(1, _, _)(2, 3))
    self.assertEqual([1, 2, 3], g(_, 2, _)(1, 3))
    self.assertEqual([1, 2, 3], g(_, _, 3)(1, 2))

    self.assertEqual([1, 2, 3], g(1, _, _)(_, 3)(2))
    self.assertEqual([1, 2, 3], g(_, 2, _)(_, 3)(1))
    self.assertEqual([1, 2, 3], g(_, _, 3)(_, 2)(1))

    self.assertEqual([1, 2, 3], g(_, _, _)(_, _)(_)(1, 2, 3))
    self.assertEqual([1, 2, 3], g(_, _, _)(1, _, _)(_, _)(2, _)(_)(3))

  def test_supports_functional_placeholder(self):
    def f(*arguments):
      return list(arguments)
    g = R.curryN(3, f)
    _ = {'@@functional/placeholder': True, 'x': randint(0, 100)}

    self.assertEqual([1, 2, 3], g(1)(2)(3))
    self.assertEqual([1, 2, 3], g(1)(2, 3))
    self.assertEqual([1, 2, 3], g(1, 2)(3))
    self.assertEqual([1, 2, 3], g(1, 2, 3))

    self.assertEqual([1, 2, 3], g(_, 2, 3)(1))
    self.assertEqual([1, 2, 3], g(1, _, 3)(2))
    self.assertEqual([1, 2, 3], g(1, 2, _)(3))

    self.assertEqual([1, 2, 3], g(1, _, _)(2)(3))
    self.assertEqual([1, 2, 3], g(_, 2, _)(1)(3))
    self.assertEqual([1, 2, 3], g(_, _, 3)(1)(2))

    self.assertEqual([1, 2, 3], g(1, _, _)(2, 3))
    self.assertEqual([1, 2, 3], g(_, 2, _)(1, 3))
    self.assertEqual([1, 2, 3], g(_, _, 3)(1, 2))

    self.assertEqual([1, 2, 3], g(1, _, _)(_, 3)(2))
    self.assertEqual([1, 2, 3], g(_, 2, _)(_, 3)(1))
    self.assertEqual([1, 2, 3], g(_, _, 3)(_, 2)(1))

    self.assertEqual([1, 2, 3], g(_, _, _)(_, _)(_)(1, 2, 3))
    self.assertEqual([1, 2, 3], g(_, _, _)(1, _, _)(_, _)(2, _)(_)(3))

  def test_forwards_extra_arguments(self):
    def f(*arguments):
      return list(arguments)
    g = R.curryN(3, f)
    self.assertEqual([1, 2, 3], g(1, 2, 3))
    self.assertEqual([1, 2, 3, 4], g(1, 2, 3, 4))
    self.assertEqual([1, 2, 3, 4], g(1, 2)(3, 4))
    self.assertEqual([1, 2, 3, 4], g(1)(2, 3, 4))
    self.assertEqual([1, 2, 3, 4], g(1)(2)(3, 4))


if __name__ == '__main__':
  unittest.main()




import unittest

import ramda as R

"""
https://github.com/ramda/ramda/blob/master/test/product.js
"""


class TestT(unittest.TestCase):
  def test_multiplies_together_the_array_of_numbers_supplied(self):
    self.assertEqual(24, R.product([1, 2, 3, 4]))


if __name__ == '__main__':
  unittest.main()



import unittest
from inspect import getfullargspec
from random import randint

import ramda as R

"""
https://github.com/ramda/ramda/blob/master/test/curry.js
"""


class TestCurry(unittest.TestCase):
  def test_curries_a_single_value(self):
    def e(a, b, c, d):
      return (a + b * c) / d
    f = R.curry(e)
    g = f(12)
    self.assertEqual(15, g(3, 6, 2))

  def test_curries_multiple_values(self):
    def e(a, b, c, d):
      return (a + b * c) / d
    f = R.curry(e)
    g = f(12, 3)
    self.assertEqual(15, g(6, 2))
    h = f(12, 3, 6)
    self.assertEqual(15, h(2))

  def test_allows_further_currying_of_a_curried_function(self):
    def e(a, b, c, d):
      return (a + b * c) / d
    f = R.curry(e)
    g = f(12)
    self.assertEqual(15, g(3, 6, 2))
    h = g(3)
    self.assertEqual(15, h(6, 2))
    self.assertEqual(15, g(3, 6)(2))

  def test_properly_reports_the_length_of_the_curried_function(self):
    def e(a, b, c, d):
      return (a + b * c) / d
    f = R.curry(e)
    fullargspec = getfullargspec(f)
    self.assertEqual(4, len(fullargspec.args))

  def test_supports_R_placeholder(self):
    def f(a, b, c):
      return [a, b, c]
    g = R.curry(f)
    _ = R.__

    self.assertEqual([1, 2, 3], g(1)(2)(3))
    self.assertEqual([1, 2, 3], g(1)(2, 3))
    self.assertEqual([1, 2, 3], g(1, 2)(3))
    self.assertEqual([1, 2, 3], g(1, 2, 3))

    self.assertEqual([1, 2, 3], g(_, 2, 3)(1))
    self.assertEqual([1, 2, 3], g(1, _, 3)(2))
    self.assertEqual([1, 2, 3], g(1, 2, _)(3))

    self.assertEqual([1, 2, 3], g(1, _, _)(2)(3))
    self.assertEqual([1, 2, 3], g(_, 2, _)(1)(3))
    self.assertEqual([1, 2, 3], g(_, _, 3)(1)(2))

    self.assertEqual([1, 2, 3], g(1, _, _)(2, 3))
    self.assertEqual([1, 2, 3], g(_, 2, _)(1, 3))
    self.assertEqual([1, 2, 3], g(_, _, 3)(1, 2))

    self.assertEqual([1, 2, 3], g(1, _, _)(_, 3)(2))
    self.assertEqual([1, 2, 3], g(_, 2, _)(_, 3)(1))
    self.assertEqual([1, 2, 3], g(_, _, 3)(_, 2)(1))

    self.assertEqual([1, 2, 3], g(_, _, _)(_, _)(_)(1, 2, 3))
    self.assertEqual([1, 2, 3], g(_, _, _)(1, _, _)(_, _)(2, _)(_)(3))

  def test_supports_functional_placeholder(self):
    def f(a, b, c):
      return [a, b, c]
    g = R.curry(f)
    _ = {'@@functional/placeholder': True, 'x': randint(0, 100)}

    self.assertEqual([1, 2, 3], g(1)(2)(3))
    self.assertEqual([1, 2, 3], g(1)(2, 3))
    self.assertEqual([1, 2, 3], g(1, 2)(3))
    self.assertEqual([1, 2, 3], g(1, 2, 3))

    self.assertEqual([1, 2, 3], g(_, 2, 3)(1))
    self.assertEqual([1, 2, 3], g(1, _, 3)(2))
    self.assertEqual([1, 2, 3], g(1, 2, _)(3))

    self.assertEqual([1, 2, 3], g(1, _, _)(2)(3))
    self.assertEqual([1, 2, 3], g(_, 2, _)(1)(3))
    self.assertEqual([1, 2, 3], g(_, _, 3)(1)(2))

    self.assertEqual([1, 2, 3], g(1, _, _)(2, 3))
    self.assertEqual([1, 2, 3], g(_, 2, _)(1, 3))
    self.assertEqual([1, 2, 3], g(_, _, 3)(1, 2))

    self.assertEqual([1, 2, 3], g(1, _, _)(_, 3)(2))
    self.assertEqual([1, 2, 3], g(_, 2, _)(_, 3)(1))
    self.assertEqual([1, 2, 3], g(_, _, 3)(_, 2)(1))

    self.assertEqual([1, 2, 3], g(_, _, _)(_, _)(_)(1, 2, 3))
    self.assertEqual([1, 2, 3], g(_, _, _)(1, _, _)(_, _)(2, _)(_)(3))

  def test_forwads_extra_arguments(self):
    def f(a, b, c, *args):
      return [a] + [b] + [c] + list(args)
    g = R.curry(f)
    self.assertEqual([1, 2, 3], g(1, 2, 3))
    self.assertEqual([1, 2, 3, 4], g(1, 2, 3, 4))
    self.assertEqual([1, 2, 3, 4], g(1, 2)(3, 4))
    self.assertEqual([1, 2, 3, 4], g(1)(2, 3, 4))
    self.assertEqual([1, 2, 3, 4], g(1)(2)(3, 4))


if __name__ == '__main__':
  unittest.main()



import unittest

import ramda as R
from ramda.private._has import _has

from .helpers.listXf import listXf

"""
https://github.com/ramda/ramda/blob/master/test/find.js
"""

obj1 = {'x': 100}
obj2 = {'x': 200}
a = [11, 10, 9, 'cow', obj1, 8, 7, 100, 200, 300, obj2, 4, 3, 2, 1, 0]
def even(x): return x % 2 == 0 if isinstance(x, int) else False
def gt100(x): return x > 100 if isinstance(x, int) else False
def isStr(x): return isinstance(x, str)
def xGt100(o): return o.get('x') > 100 if _has(o, 'get') else False


class TestFind(unittest.TestCase):
  def test_returns_the_first_elemenet_that_satisfies_the_predicate(self):
    self.assertEqual(10, R.find(even, a))
    self.assertEqual(200, R.find(gt100, a))
    self.assertEqual('cow', R.find(isStr, a))
    self.assertEqual(obj2, R.find(xGt100, a))

  def test_returns_None_when_no_element_statisfies_the_predicate(self):
    self.assertEqual(None, R.find(even, ['zing']))

  def test_returns_None_when_given_an_empty_list(self):
    self.assertEqual(None, R.find(even, []))

  def test_dispatches_to_transformer_objects(self):
    res = R.find(R.identity, listXf)
    self.assertEqual(res.f, R.identity)
    self.assertEqual(res.found, False)
    self.assertEqual(res.xf, listXf)

  def test_can_act_as_a_transducer(self):
    self.assertEqual([10], R.into([], R.find(even), a))
    # TODO: R.transduce


if __name__ == '__main__':
  unittest.main()



from ._helper import getAttribute
from ._xfBase import XfBase


class XDrop(XfBase):
  def __init__(self, n, xf):
    self.xf = xf
    self.n = n

  def step(self, result, _input):
    if self.n > 0:
      self.n -= 1
      return result
    return getAttribute(self.xf, '@@transducer/step')(result, _input)


def _xdrop(n): return lambda xf: XDrop(n, xf)



from .private._clone import _clone
from .private._curry1 import _curry1
from .private._helper import getAttribute
from .private._isFunction import _isFunction


def inner_clone(value):
  if _isFunction(getAttribute(value, 'clone')):
    return value.clone()
  return _clone(value, True)


clone = _curry1(inner_clone)



import unittest

import ramda as R

from .helpers.Maybe import Just, Nothing

"""
https://github.com/ramda/ramda/blob/master/test/empty.js
"""


class TestEmpty(unittest.TestCase):
  def test_dispatches_to_empty_method(self):
    self.assertTrue(R.empty(Nothing()).isNothing)
    self.assertTrue(R.empty(Just([42])).isNothing)

  def test_returns_empty_dict_given_dict(self):
    self.assertEqual({}, R.empty({'x': 1, 'y': 2}))

  def test_returns_empty_array_given_array(self):
    self.assertEqual([], R.empty([1, 2, 3]))

  def test_returns_empty_string_given_string(self):
    self.assertEqual('', R.empty('abc'))

  def test_returns_empty_set_given_set(self):
    self.assertEqual(set(), R.empty(set([1, 2, 3])))

  def test_returns_empty_None_given_None(self):
    self.assertEqual(None, R.empty(None))

  def test_returns_origin_object(self):
    class Obj:
      def __init__(self, value):
        self.value = value
    o = Obj(42)
    self.assertTrue(isinstance(R.empty(o), Obj))
    self.assertEqual(42, R.empty(o).value)


if __name__ == '__main__':
  unittest.main()



import unittest

import ramda as R

"""
https://github.com/ramda/ramda/blob/master/test/differenceWith.js
"""

Ro = [{'a': 1}, {'a': 2}, {'a': 3}, {'a': 4}]
Ro2 = [{'a': 1}, {'a': 2}, {'a': 3}, {'a': 4}, {'a': 1}, {'a': 2}, {'a': 3}, {'a': 4}]
So = [{'a': 3}, {'a': 4}, {'a': 5}, {'a': 6}]
So2 = [{'a': 3}, {'a': 4}, {'a': 5}, {'a': 6}, {'a': 3}, {'a': 4}, {'a': 5}, {'a': 6}]
def eqA(r, s): return r['a'] == s['a']
def identical(a, b): return a == b


class TestDifferenceWith(unittest.TestCase):
  def test_combines_two_lists_into_the_set_of_all_their_elements_based_on_the_pased_in_equality_predicate(self):
    self.assertEqual([{'a': 1}, {'a': 2}], R.differenceWith(eqA, Ro, So))

  def test_does_not_allow_duplicates_in_the_output_even_if_the_input_lists_had_duplicates(self):
    self.assertEqual([{'a': 1}, {'a': 2}], R.differenceWith(eqA, Ro2, So2))

if __name__ == '__main__':
  unittest.main()



from .private._curry2 import _curry2
from .private._dispatchable import _dispatchable
from .private._Set import _Set
from .private._xuniqBy import _xuniqBy


def inner_uniqBy(fn, arr):
  _set = _Set()
  result = []
  idx = 0

  while idx < len(arr):
    item = arr[idx]
    appliedItem = fn(item)
    if _set.add(appliedItem):
      print(appliedItem, _set)
      result.append(item)
    idx += 1
  return result


uniqBy = _curry2(_dispatchable(['uniqBy'], _xuniqBy, inner_uniqBy))




import unittest

import ramda as R

"""
https://github.com/ramda/ramda/blob/master/test/reduceBy.js
"""

byType = R.prop('type')
def sumValues(acc, obj): return acc + obj['val']


sumInput = [
    {'type': 'A', 'val': 10},
    {'type': 'B', 'val': 20},
    {'type': 'A', 'val': 30},
    {'type': 'A', 'val': 40},
    {'type': 'C', 'val': 50},
    {'type': 'B', 'val': 60}
]


def grade(score):
  if score < 65:
    return 'F'
  elif score < 70:
    return 'D'
  elif score < 80:
    return 'C'
  elif score < 90:
    return 'B'
  else:
    return 'A'


students = [
    {'name': 'Abby', 'score': 84},
    {'name': 'Brad', 'score': 73},
    {'name': 'Chris', 'score': 89},
    {'name': 'Dianne', 'score': 99},
    {'name': 'Eddy', 'score': 58},
    {'name': 'Fred', 'score': 67},
    {'name': 'Gillian', 'score': 91},
    {'name': 'Hannah', 'score': 78},
    {'name': 'Irene', 'score': 85},
    {'name': 'Jack', 'score': 69}
]


def byGrade(student):
  return grade(student['score'] or 0)


class TestReduceBy(unittest.TestCase):
  def test_splits_the_list_into_groups_according_to_the_grouping_function(self):
    def collectNames(acc, student): return acc + [student['name']]
    self.assertEqual({
        'A': ['Dianne', 'Gillian'],
        'B': ['Abby', 'Chris', 'Irene'],
        'C': ['Brad', 'Hannah'],
        'D': ['Fred', 'Jack'],
        'F': ['Eddy']
    }, R.reduceBy(collectNames, [], byGrade, students))

  def test_splits_the_list_into_mutation_free_groups(self):
    def collectNames(acc, student):
      acc.append(student['name'])
      return acc
    self.assertEqual({
        'A': ['Dianne', 'Gillian'],
        'B': ['Abby', 'Chris', 'Irene'],
        'C': ['Brad', 'Hannah'],
        'D': ['Fred', 'Jack'],
        'F': ['Eddy']
    }, R.reduceBy(collectNames, [], byGrade, students))

  def test_returns_an_empty_object_if_given_an_empty_array(self):
    self.assertEqual({}, R.reduceBy(sumValues, 0, byType, []))

  def test_can_act_as_a_transducer(self):
    reduceToSumByType = R.reduceBy(sumValues, 0)
    sumByType = reduceToSumByType(byType)
    self.assertEqual(
        {'A': 800, 'B': 800, 'C': 500},
        R.into(
            {},
            R.compose(sumByType, R.map(R.adjust(1, R.multiply(10)))),
            sumInput)
    )

  def test_short_circuits_with_reduced(self):
    def collectNames(acc, student): return R.reduced(acc) if student['name'] == 'Fred' else acc + [student['name']]
    self.assertEqual({
        'A': ['Dianne'],
        'B': ['Abby', 'Chris'],
        'C': ['Brad'],
        'F': ['Eddy']
    }, R.reduceBy(collectNames, [], byGrade, students))
    # TODO: R.transduce


if __name__ == '__main__':
  unittest.main()



import unittest
from datetime import date
from math import isnan

import ramda as R

"""
https://github.com/ramda/ramda/blob/master/test/add.js
"""


class TestAdd(unittest.TestCase):
  def test_adds_together_two_numbers(self):
    self.assertEqual(10, R.add(3, 7))

  def test_coerces_its_arguments_to_numbers(self):
    self.assertEqual(3, R.add('1', '2'))
    self.assertEqual(3, R.add(1, '2'))
    self.assertEqual(1, R.add(True, False))
    self.assertTrue(isnan(R.add(None, None)))
    self.assertTrue(isnan(R.add(date(1, 1, 1), date(2, 1, 1))))

  def test_float(self):
    self.assertEqual(4.2, R.add(1.1, 3.1))
    self.assertEqual(4.2, R.add('1.1', 3.1))
    self.assertEqual(4.2, R.add('1.1', '3.1'))


class TestAddProperties(unittest.TestCase):
  def test_commutative(self):
    self.assertEqual(R.add(1, 2), R.add(2, 1))

  def test_associative(self):
    self.assertEqual(R.add(1, R.add(2, 3)), R.add(R.add(1, 2), 3))

  def test_identity(self):
    self.assertEqual(R.add(1, 0), 1)
    self.assertEqual(1, R.add(0, 1))


if __name__ == '__main__':
  unittest.main()



import unittest

import ramda as R

"""
https://github.com/ramda/ramda/blob/master/test/countBy.js
"""

albums = [
    {"title": 'Art of the Fugue', "artist": 'Glenn Gould', "genre": 'Baroque'},
    {"title": 'A Farewell to Kings', "artist": 'Rush', "genre": 'Rock'},
    {"title": 'Timeout', "artist": 'Dave Brubeck Quartet', "genre": 'Jazz'},
    {"title": 'Fly By Night', "artist": 'Rush', "genre": 'Rock'},
    {"title": 'Goldberg Variations', "artist": 'Daniel Barenboim', "genre": 'Baroque'},
    {"title": 'New World Symphony', "artist": 'Leonard Bernstein', "genre": 'Romantic'},
    {"title": 'Romance with the Unseen', "artist": 'Don Byron', "genre": 'Jazz'},
    {"title": 'Somewhere In Time', "artist": 'Iron Maiden', "genre": 'Metal'},
    {"title": 'In Times of Desparation', "artist": 'Danny Holt', "genre": 'Modern'},
    {"title": 'Evita', "artist": 'Various', "genre": 'Broadway'},
    {"title": 'Five Leaves Left', "artist": 'Nick Drake', "genre": 'Folk'},
    {"title": 'The Magic Flute', "artist": 'John Eliot Gardiner', "genre": 'Classical'}
]


def deriveGenre():
  remap = {
      "Baroque": 'Classical',
      "Modern": 'Classical',
      "Romantic": 'Classical',
      "Metal": 'Rock'
  }

  def f(album):
    genre = R.prop('genre', album)
    return remap.get(genre) or genre
  return f


derivedGenre = deriveGenre()


class TestCountBy(unittest.TestCase):
  def test_counts_by_a_simple_property_of_the_objects(self):
    self.assertEqual(
        {"Baroque": 2, "Rock": 2, "Jazz": 2, "Romantic": 1, "Metal": 1, "Modern": 1, "Broadway": 1, "Folk": 1, "Classical": 1},
        R.countBy(R.prop('genre'), albums)
    )

  def test_counts_by_a_more_complex_function_on_the_objects(self):
    self.assertEqual(
        {"Classical": 5, "Rock": 3, "Jazz": 2, "Broadway": 1, "Folk": 1},
        R.countBy(derivedGenre, albums)
    )

  def test_ignores_inherited_properties(self):
    result = R.countBy(R.identity, ['abc', '__dict__'])
    self.assertEqual(1, result['abc'])
    self.assertEqual(1, result['__dict__'])

  def test_can_act_as_transducer(self):
    transducer = R.compose(
        R.countBy(R.prop('genre')),
        R.map(R.adjust(1, lambda x: -x))
    )
    expected = {"Baroque": -2, "Rock": -2, "Jazz": -2, "Romantic": -1, "Metal": -1, "Modern": -1, "Broadway": -1, "Folk": -1, "Classical": -1}
    self.assertEqual(expected, R.into({}, transducer, albums))
    # TODO: R.transduce


if __name__ == '__main__':
  unittest.main()



from .private._curry2 import _curry2
from .toString import toString


def inner_min(a, b):
  if a == b:
    return a

  # pylint: disable=unidiomatic-typecheck
  if type(a) != type(b):
    strA = str(a)
    strB = str(b)
    return a if strA < strB else b

  def safeMin(x, y):
    if (x > y) != (y > x):
      return x if x < y else y
    return None
  minByValue = safeMin(a, b)
  if minByValue is not None:
    return minByValue

  stringA = toString(a)
  minByStringValue = safeMin(stringA, toString(b))
  if minByStringValue is not None:
    return a if minByStringValue == stringA else b
  return a


Min = _curry2(inner_min)



from ._helper import getAttribute
from ._isArray import _isArray
from ._isFunction import _isFunction


def _checkForMethod(methodName, fn):
  def wrapper(*arguments):
    if len(arguments) == 0:
      return fn()
    obj = arguments[-1]
    if _isArray(obj) or not _isFunction(getAttribute(obj, methodName)):
      return fn(*arguments)
    return getAttribute(obj, methodName)(*arguments[:-1])
  return wrapper
